@@ -12,14 +12,14 @@ set -e
 b=${1:?"missing base directory name argument"}
 tar -cf $b-binary.tar \
     $b/output/sbcl.core $b/src/runtime/sbcl \
-    $b/BUGS $b/COPYING $b/CREDITS $b/INSTALL $b/NEWS $b/README $b/SUPPORT \
-    $b/install.sh $b/find-gnumake.sh $b/sbcl-pwd.sh \
+    $b/BUGS $b/COPYING $b/CREDITS $b/INSTALL $b/NEWS $b/README \
+    $b/install.sh $b/find-gnumake.sh $b/sbcl-pwd.sh $b/run-sbcl.sh \
     $b/doc/sbcl.1 \
     $b/pubring.pgp \
     $b/contrib/asdf-module.mk \
     $b/contrib/vanilla-module.mk \
     `for dir in $b/contrib/*; do
          if test -d $dir && test -f $dir/test-passed; then
-             find $dir -name CVS -type d -prune -o -not -type d -not -name '.cvsignore' -print
+             find $dir -name CVS -type d -prune -o \! -type d \! -name '.cvsignore' -print
          fi
      done`
