@@ -48,6 +48,16 @@ int main(int argc, char *argv[]) {
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for AF_LOCAL (unknown to the C compiler).\")\n");
 #endif
+#ifdef AF_INET6
+    fprintf (out, "(cl:defconstant AF-INET6 %d \"IP version 6\")\n", AF_INET6);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for AF_INET6 (unknown to the C compiler).\")\n");
+#endif
+#ifdef AF_NETLINK
+    fprintf (out, "(cl:defconstant AF-ROUTE %d \"Alias to emulate 4.4BSD \")\n", AF_NETLINK);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for AF_NETLINK (unknown to the C compiler).\")\n");
+#endif
 #ifdef SOCK_STREAM
     fprintf (out, "(cl:defconstant SOCK-STREAM %d \"Sequenced, reliable, connection-based byte streams.\")\n", SOCK_STREAM);
 #else
@@ -113,6 +123,11 @@ int main(int argc, char *argv[]) {
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_SNDBUF (unknown to the C compiler).\")\n");
 #endif
+#ifdef SO_PASSCRED
+    fprintf (out, "(cl:defconstant SO-PASSCRED %d \"NIL\")\n", SO_PASSCRED);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_PASSCRED (unknown to the C compiler).\")\n");
+#endif
 #ifdef SO_RCVBUF
     fprintf (out, "(cl:defconstant SO-RCVBUF %d \"NIL\")\n", SO_RCVBUF);
 #else
@@ -133,11 +148,21 @@ int main(int argc, char *argv[]) {
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_NO_CHECK (unknown to the C compiler).\")\n");
 #endif
+#ifdef SO_PRIORITY
+    fprintf (out, "(cl:defconstant SO-PRIORITY %d \"NIL\")\n", SO_PRIORITY);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_PRIORITY (unknown to the C compiler).\")\n");
+#endif
 #ifdef SO_LINGER
     fprintf (out, "(cl:defconstant SO-LINGER %d \"For reliable streams, pause a while on closing when unsent messages are queued\")\n", SO_LINGER);
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_LINGER (unknown to the C compiler).\")\n");
 #endif
+#ifdef SO_BSDCOMPAT
+    fprintf (out, "(cl:defconstant SO-BSDCOMPAT %d \"NIL\")\n", SO_BSDCOMPAT);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_BSDCOMPAT (unknown to the C compiler).\")\n");
+#endif
 #ifdef SO_SNDLOWAT
     fprintf (out, "(cl:defconstant SO-SNDLOWAT %d \"NIL\")\n", SO_SNDLOWAT);
 #else
@@ -163,6 +188,11 @@ int main(int argc, char *argv[]) {
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for TCP_NODELAY (unknown to the C compiler).\")\n");
 #endif
+#ifdef SO_BINDTODEVICE
+    fprintf (out, "(cl:defconstant SO-BINDTODEVICE %d \"NIL\")\n", SO_BINDTODEVICE);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for SO_BINDTODEVICE (unknown to the C compiler).\")\n");
+#endif
 #ifdef IFNAMSIZ
     fprintf (out, "(cl:defconstant IFNAMSIZ %d \"NIL\")\n", IFNAMSIZ);
 #else
@@ -335,6 +365,16 @@ int main(int argc, char *argv[]) {
     fprintf (out, "(cl:defconstant MSG-NOSIGNAL %d \"NIL\")\n", MSG_NOSIGNAL);
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for MSG_NOSIGNAL (unknown to the C compiler).\")\n");
+#endif
+#ifdef MSG_CONFIRM
+    fprintf (out, "(cl:defconstant MSG-CONFIRM %d \"NIL\")\n", MSG_CONFIRM);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for MSG_CONFIRM (unknown to the C compiler).\")\n");
+#endif
+#ifdef MSG_MORE
+    fprintf (out, "(cl:defconstant MSG-MORE %d \"NIL\")\n", MSG_MORE);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for MSG_MORE (unknown to the C compiler).\")\n");
 #endif
     fprintf (out, "(cl:eval-when (:compile-toplevel :load-toplevel :execute) (sb-alien:define-alien-type SOCKLEN-T (sb-alien:%ssigned %d)))\n", SIGNED_(socklen_t), (8*sizeof(socklen_t)));
     fprintf (out, "(cl:eval-when (:compile-toplevel :load-toplevel :execute) (sb-alien:define-alien-type SIZE-T (sb-alien:%ssigned %d)))\n", SIGNED_(size_t), (8*sizeof(size_t)));
@@ -383,13 +423,6 @@ int main(int argc, char *argv[]) {
 }
     fprintf (out, "))\n");
     fprintf (out, "(cl:eval-when (:compile-toplevel :load-toplevel :execute) (sb-grovel::define-c-struct SOCKADDR-IN %d\n", sizeof(struct sockaddr_in));
-    fprintf (out, " (LEN (UNSIGNED 8) \"__uint8_t\"\n");
-{ struct sockaddr_in t;
-    fprintf (out, "  %d\n", ((unsigned long)&(t.sin_len)) - ((unsigned long)&(t)));
-}
-{ struct sockaddr_in t;
-    fprintf (out, "  %d)\n", sizeof(t.sin_len));
-}
     fprintf (out, " (FAMILY INTEGER \"sa_family_t\"\n");
 { struct sockaddr_in t;
     fprintf (out, "  %d\n", ((unsigned long)&(t.sin_family)) - ((unsigned long)&(t)));
@@ -424,6 +457,22 @@ int main(int argc, char *argv[]) {
 { struct sockaddr_un t;
     fprintf (out, "  %d\n", ((unsigned long)&(t.sun_path)) - ((unsigned long)&(t)));
 }
+{ struct sockaddr_un t;
+    fprintf (out, "  %d)\n", sizeof(t.sun_path));
+}
+    fprintf (out, "))\n");
+    fprintf (out, "(cl:eval-when (:compile-toplevel :load-toplevel :execute) (sb-grovel::define-c-struct SOCKADDR-UN-ABSTRACT %d\n", sizeof(struct sockaddr_un));
+    fprintf (out, " (FAMILY INTEGER \"sa_family_t\"\n");
+{ struct sockaddr_un t;
+    fprintf (out, "  %d\n", ((unsigned long)&(t.sun_family)) - ((unsigned long)&(t)));
+}
+{ struct sockaddr_un t;
+    fprintf (out, "  %d)\n", sizeof(t.sun_family));
+}
+    fprintf (out, " (PATH (ARRAY (UNSIGNED 8)) \"char\"\n");
+{ struct sockaddr_un t;
+    fprintf (out, "  %d\n", ((unsigned long)&(t.sun_path)) - ((unsigned long)&(t)));
+}
 { struct sockaddr_un t;
     fprintf (out, "  %d)\n", sizeof(t.sun_path));
 }
@@ -558,12 +607,12 @@ int main(int argc, char *argv[]) {
     fprintf (out, "(cl:declaim (cl:inline GETHOSTBYNAME))\n");
     fprintf (out, "(sb-grovel::define-foreign-routine (\"gethostbyname\" GETHOSTBYNAME)\n");
     fprintf (out, "  (* HOSTENT)\n  (NAME C-STRING))\n");
-    fprintf (out, "(cl:declaim (cl:inline GETHOSTBYNAME2))\n");
-    fprintf (out, "(sb-grovel::define-foreign-routine (\"gethostbyname2\" GETHOSTBYNAME2)\n");
-    fprintf (out, "  (* HOSTENT)\n  (NAME C-STRING)\n  (AF INT))\n");
     fprintf (out, "(cl:declaim (cl:inline GETHOSTBYADDR))\n");
     fprintf (out, "(sb-grovel::define-foreign-routine (\"gethostbyaddr\" GETHOSTBYADDR)\n");
     fprintf (out, "  (* HOSTENT)\n  (ADDR (* T))\n  (LEN INT)\n  (AF INT))\n");
+    fprintf (out, "(cl:declaim (cl:inline GETHOSTBYNAME-R))\n");
+    fprintf (out, "(sb-grovel::define-foreign-routine (\"gethostbyname_r\" GETHOSTBYNAME-R)\n");
+    fprintf (out, "  INT\n  (NAME C-STRING)\n  (RET (* HOSTENT))\n  (BUF (* CHAR))\n  (BUFLEN LONG)\n  (RESULT (* (* HOSTENT)))\n  (H-ERRNOP (* INT)))\n");
     fprintf (out, "(cl:eval-when (:compile-toplevel :load-toplevel :execute) (sb-grovel::define-c-struct ADDRINFO %d\n", sizeof(struct addrinfo));
     fprintf (out, " (FLAGS INTEGER \"int\"\n");
 { struct addrinfo t;
@@ -593,7 +642,7 @@ int main(int argc, char *argv[]) {
 { struct addrinfo t;
     fprintf (out, "  %d)\n", sizeof(t.ai_protocol));
 }
-    fprintf (out, " (ADDRLEN INTEGER \"socklen_t\"\n");
+    fprintf (out, " (ADDRLEN INTEGER \"size_t\"\n");
 { struct addrinfo t;
     fprintf (out, "  %d\n", ((unsigned long)&(t.ai_addrlen)) - ((unsigned long)&(t)));
 }
