@@ -29,7 +29,13 @@ export LANG LC_ALL
 # system with the :SB-SHOW feature enabled, it does it rather silently,
 # without trying to tell you about what it's doing. So unless it hangs
 # for much longer than that, don't worry, it's likely to be normal.
-echo //doing warm init
+echo //doing warm init - compilation phase
 ./src/runtime/sbcl \
 --core output/cold-sbcl.core \
+--lose-on-corruption \
 --no-sysinit --no-userinit < make-target-2.lisp
+echo //doing warm init - load and dump phase
+./src/runtime/sbcl \
+--core output/cold-sbcl.core \
+--lose-on-corruption \
+--no-sysinit --no-userinit < make-target-2-load.lisp
