@@ -24,7 +24,7 @@
     (/primitive-print (symbol-name name))
     (when trans-p
       (/show0 "in TRANS-P case")
-      (let ((classoid (classoid-cell-classoid (find-classoid-cell name)))
+      (let ((classoid (classoid-cell-classoid (find-classoid-cell name :create t)))
             (type (specifier-type translation)))
         (setf (built-in-classoid-translation classoid) type)
         (setf (info :type :builtin name) type)))))
