@@ -15,6 +15,9 @@
   (declare (ignore name value))
   nil)
 
+#!+(and sb-lutex sb-thread)
+(defun make-lutex () nil)
+
 (defmacro with-mutex ((mutex) &body body)
   (declare (ignore mutex))
   `(locally ,@body))
@@ -29,7 +32,7 @@
 
 (defun get-spinlock (spinlock)
   (declare (ignore spinlock))
-  nil)
+  t)
 
 (defun release-spinlock (spinlock)
   (declare (ignore spinlock))
