@@ -26,7 +26,6 @@
   (unparse-file (missing-arg) :type function)
   (unparse-enough (missing-arg) :type function)
   (unparse-directory-separator (missing-arg) :type simple-string)
-  (simplify-namestring (missing-arg) :type function)
   (customary-case (missing-arg) :type (member :upper :lower)))
 
 (def!method print-object ((host host) stream)
@@ -37,14 +36,14 @@
              (:include host
                        (parse #'parse-logical-namestring)
                        (parse-native
-                        (lambda (x)
+                        (lambda (&rest x)
                           (error "called PARSE-NATIVE-NAMESTRING using a ~
-                                  logical host: ~S" x)))
+                                  logical host: ~S" (first x))))
                        (unparse #'unparse-logical-namestring)
                        (unparse-native
-                        (lambda (x)
+                        (lambda (&rest x)
                           (error "called NATIVE-NAMESTRING using a ~
-                                  logical host: ~S" x)))
+                                  logical host: ~S" (first x))))
                        (unparse-host
                         (lambda (x)
                           (logical-host-name (%pathname-host x))))
@@ -52,7 +51,6 @@
                        (unparse-file #'unparse-logical-file)
                        (unparse-enough #'unparse-enough-namestring)
                        (unparse-directory-separator ";")
-                       (simplify-namestring #'identity)
                        (customary-case :upper)))
   (name "" :type simple-string)
   (translations nil :type list)
