@@ -119,3 +119,8 @@ exit(3) directly will circumvent these hooks.")
                   (%shrink-vector string size)
                   string)))
          ,@body))))
+
+;;; The smallest power of two that is equal to or greater than X.
+(defun power-of-two-ceiling (x)
+  (declare (index x))
+  (ash 1 (integer-length (1- x))))
