@@ -564,7 +564,7 @@
 #!-sb-fluid
 (declaim (inline get-time-of-day))
 (defun get-time-of-day ()
-  "Return the number of seconds and microseconds since the beginning og the
+  "Return the number of seconds and microseconds since the beginning of the
 UNIX epoch: January 1st 1970."
   (with-alien ((system-time filetime))
     (syscall (("GetSystemTimeAsFileTime" 4) void (* filetime))
