@@ -81,6 +81,11 @@
 ;;; of symbols with thread local bindings.
 (def!constant tls-size 4096)
 
+#!+gencgc
+(progn
+  (def!constant +highest-normal-generation+ 5)
+  (def!constant +pseudo-static-generation+ 6))
+
 (defenum ()
   trace-table-normal
   trace-table-call-site
