@@ -118,7 +118,7 @@
             #-no-ansi-print-object
             (:print-object (lambda (x s)
                              (print-unreadable-object (x s :type t)
-                               (prin1 (class-info-name x)))))
+                               (prin1 (class-info-name x) s))))
             (:copier nil))
   ;; name of this class
   (name nil :type keyword :read-only t)
@@ -344,7 +344,6 @@
 
 ;;;; generic interfaces
 
-;;; FIXME: used only in this file, needn't be in runtime
 (defmacro do-info ((env &key (name (gensym)) (class (gensym)) (type (gensym))
                         (type-number (gensym)) (value (gensym)) known-volatile)
                    &body body)
@@ -1033,12 +1032,6 @@
   :type-spec (or fun-info null)
   :default nil)
 
-(define-info-type
-  :class :function
-  :type :documentation
-  :type-spec (or string null)
-  :default nil)
-
 (define-info-type
   :class :function
   :type :definition
@@ -1206,6 +1199,10 @@
                        (when (info :type :kind name)
                          (error 'declaration-type-conflict-error
                                 :format-arguments (list name)))))
+(define-info-type
+  :class :declaration
+  :type :handler
+  :type-spec (or function null))
 
 (define-info-class :alien-type)
 (define-info-type
