@@ -60,7 +60,7 @@
     (loadw value obj-temp symbol-value-slot other-pointer-lowtag)
     (let ((err-lab (generate-error-code vop unbound-symbol-error obj-temp)))
       (inst xor temp value unbound-marker-widetag)
-      (inst beq temp zero-tn err-lab)
+      (inst beq temp err-lab)
       (inst nop))))
 
 ;;; Like CHECKED-CELL-REF, only we are a predicate to see if the cell is bound.
@@ -78,8 +78,8 @@
     (loadw value object symbol-value-slot other-pointer-lowtag)
     (inst xor temp value unbound-marker-widetag)
     (if not-p
-        (inst beq temp zero-tn target)
-        (inst bne temp zero-tn target))
+        (inst beq temp target)
+        (inst bne temp target))
     (inst nop)))
 
 (define-vop (fast-symbol-value cell-ref)
@@ -91,19 +91,24 @@
   (:policy :fast-safe)
   (:translate symbol-hash)
   (:args (symbol :scs (descriptor-reg)))
+  (:temporary (:scs (non-descriptor-reg)) temp)
   (:results (res :scs (any-reg)))
   (:result-types positive-fixnum)
-  (:temporary (:scs (any-reg)) temp)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
     ;; cdr slot, so we have to strip off the two low bits to make sure
     ;; it is a fixnum.  The lowtag selection magic that is required to
     ;; ensure this is explained in the comment in objdef.lisp
-    ;;
-    ;; wow, MIPS sucks (or I do) -- CSR, 2004-05-20
-    (inst li temp (fixnumize -1))
-    (loadw res symbol symbol-hash-slot other-pointer-lowtag)
-    (inst and res temp)))
+    (loadw temp symbol symbol-hash-slot other-pointer-lowtag)
+    (inst srl temp n-fixnum-tag-bits)
+    (inst sll res temp n-fixnum-tag-bits)))
+
+;;; On unithreaded builds these are just copies of the non-global versions.
+(define-vop (%set-symbol-global-value set))
+(define-vop (symbol-global-value symbol-value)
+  (:translate symbol-global-value))
+(define-vop (fast-symbol-global-value fast-symbol-value)
+  (:translate symbol-global-value))
 
 ;;;; Fdefinition (fdefn) objects.
 
@@ -136,7 +141,7 @@
       (load-type type function (- fun-pointer-lowtag))
       (inst nop)
       (inst xor type simple-fun-header-widetag)
-      (inst beq type zero-tn normal-fn)
+      (inst beq type normal-fn)
       (inst addu lip function
             (- (ash simple-fun-code-offset word-shift)
                fun-pointer-lowtag))
@@ -203,7 +208,7 @@
 
       (emit-label loop)
       (loadw symbol bsp-tn (- binding-symbol-slot binding-size))
-      (inst beq symbol zero-tn skip)
+      (inst beq symbol skip)
       (loadw value bsp-tn (- binding-value-slot binding-size))
       (storew value symbol symbol-value-slot other-pointer-lowtag)
       (storew zero-tn bsp-tn (- binding-symbol-slot binding-size))
@@ -294,7 +299,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset n-word-bytes)
     (inst addu lip offset object)
@@ -315,7 +320,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset n-word-bytes)
     (inst addu lip offset object)
@@ -336,7 +341,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset n-word-bytes)
     (inst addu lip offset object)
@@ -357,7 +362,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset n-word-bytes)
     (inst addu lip offset object)
@@ -379,7 +384,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 2 n-word-bytes))
     (inst addu lip offset object)
@@ -408,7 +413,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 2 n-word-bytes))
     (inst addu lip offset object)
@@ -438,7 +443,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 2 n-word-bytes))
     (inst addu lip offset object)
@@ -467,7 +472,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 2 n-word-bytes))
     (inst addu lip offset object)
@@ -503,7 +508,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 4 n-word-bytes))
     (inst addu lip offset object)
@@ -566,7 +571,7 @@
   (:generator 5
     (loadw offset object 0 instance-pointer-lowtag)
     (inst srl offset n-widetag-bits)
-    (inst sll offset 2)
+    (inst sll offset n-fixnum-tag-bits)
     (inst subu offset index)
     (inst subu offset (* 4 n-word-bytes))
     (inst addu lip offset object)
