@@ -284,6 +284,14 @@
      (if (static-symbol-p value)
          (sc-number-or-lose 'immediate)
          nil))))
+
+;;; A predicate to see if a character can be used as an inline
+;;; constant (the immediate field in the instruction used is sixteen
+;;; bits wide, which is not the same as any defined subtype of
+;;; CHARACTER).
+(defun inlinable-character-constant-p (char)
+  (and (characterp char)
+       (< (char-code char) #x10000)))
 
 ;;;; function call parameters
 
