@@ -291,7 +291,7 @@
              (or end length)
              (sequence-bounding-indices-bad-error vector start end)))))
 
-(deftype eq-comparable-type ()
+(def!type eq-comparable-type ()
   '(or fixnum (not number)))
 
 ;;; True if EQL comparisons involving type can be simplified to EQ.
@@ -518,9 +518,9 @@
                     (vector t &key (:start t) (:end t))
                     *
                     :node node)
-  (let* ((element-ctype (extract-upgraded-element-type seq))
+  (let* ((type (lvar-type seq))
+         (element-ctype (array-type-upgraded-element-type type))
          (element-type (type-specifier element-ctype))
-         (type (lvar-type seq))
          (saetp (unless (eq *wild-type* element-ctype)
                   (find-saetp-by-ctype element-ctype))))
     (cond ((eq *wild-type* element-ctype)
@@ -776,7 +776,7 @@
                (end1 (or end1 len1))
                (end2 (or end2 len2))
                (replace-len (min (- end1 start1) (- end2 start2))))
-          ,(unless (policy node (= safety 0))
+          ,(unless (policy node (= insert-array-bounds-checks 0))
              `(progn
                 (unless (<= 0 start1 end1 len1)
                   (sequence-bounding-indices-bad-error seq1 start1 end1))
@@ -1082,62 +1082,71 @@
 ;;; this transform to non-strings, but I chose to just do the case that
 ;;; should cover 95% of CONCATENATE performance complaints for now.
 ;;;   -- JES, 2007-11-17
+;;;
+;;; Only handle the simple result type cases. If somebody does (CONCATENATE
+;;; '(STRING 6) ...) their code won't be optimized, but nobody does that in
+;;; practice.
 (deftransform concatenate ((result-type &rest lvars)
-                           (symbol &rest sequence)
-                           *
-                           :policy (> speed space))
-  (unless (constant-lvar-p result-type)
-    (give-up-ir1-transform))
-  (let* ((element-type (let ((type (lvar-value result-type)))
-                         ;; Only handle the simple result type cases. If
-                         ;; somebody does (CONCATENATE '(STRING 6) ...)
-                         ;; their code won't be optimized, but nobody does
-                         ;; that in practice.
-                         (case type
-                           ((string simple-string) 'character)
-                           ((base-string simple-base-string) 'base-char)
-                           (t (give-up-ir1-transform)))))
-         (vars (loop for x in lvars collect (gensym)))
-         (lvar-values (loop for lvar in lvars
-                            collect (when (constant-lvar-p lvar)
-                                      (lvar-value lvar))))
-         (lengths
-          (loop for value in lvar-values
-                for var in vars
-                collect (if value
-                            (length value)
-                            `(sb!impl::string-dispatch ((simple-array * (*))
-                                                        sequence)
-                                 ,var
-                               (declare (muffle-conditions compiler-note))
-                               (length ,var))))))
-    `(apply
-      (lambda ,vars
-        (declare (ignorable ,@vars))
-        (let* ((.length. (+ ,@lengths))
-               (.pos. 0)
-               (.string. (make-string .length. :element-type ',element-type)))
-          (declare (type index .length. .pos.)
-                   (muffle-conditions compiler-note))
-          ,@(loop for value in lvar-values
-                  for var in vars
-                  collect (if (stringp value)
-                              ;; Fold the array reads for constant arguments
-                              `(progn
-                                 ,@(loop for c across value
-                                         collect `(setf (aref .string.
-                                                              .pos.) ,c)
-                                         collect `(incf .pos.)))
-                              `(sb!impl::string-dispatch
-                                   (#!+sb-unicode
-                                    (simple-array character (*))
-                                    (simple-array base-char (*))
-                                    t)
-                                   ,var
-                                 (replace .string. ,var :start1 .pos.)
-                                 (incf .pos. (length ,var)))))
-          .string.))
-      lvars)))
+                           ((constant-arg
+                             (member string simple-string base-string simple-base-string))
+                            &rest sequence)
+                           * :node node)
+  (let ((vars (loop for x in lvars collect (gensym)))
+        (type (lvar-value result-type)))
+    (if (policy node (<= speed space))
+        ;; Out-of-line
+        `(lambda (.dummy. ,@vars)
+           (declare (ignore .dummy.))
+           ,(ecase type
+                   ((string simple-string)
+                    `(%concatenate-to-string ,@vars))
+                   ((base-string simple-base-string)
+                    `(%concatenate-to-base-string ,@vars))))
+        ;; Inline
+        (let* ((element-type (ecase type
+                               ((string simple-string) 'character)
+                               ((base-string simple-base-string) 'base-char)))
+               (lvar-values (loop for lvar in lvars
+                                  collect (when (constant-lvar-p lvar)
+                                            (lvar-value lvar))))
+               (lengths
+                (loop for value in lvar-values
+                      for var in vars
+                      collect (if value
+                                  (length value)
+                                  `(sb!impl::string-dispatch ((simple-array * (*))
+                                                              sequence)
+                                       ,var
+                                     (declare (muffle-conditions compiler-note))
+                                     (length ,var))))))
+          `(apply
+            (lambda ,vars
+              (declare (ignorable ,@vars))
+              (declare (optimize (insert-array-bounds-checks 0)))
+              (let* ((.length. (+ ,@lengths))
+                     (.pos. 0)
+                     (.string. (make-string .length. :element-type ',element-type)))
+                (declare (type index .length. .pos.)
+                         (muffle-conditions compiler-note))
+                ,@(loop for value in lvar-values
+                        for var in vars
+                        collect (if (stringp value)
+                                    ;; Fold the array reads for constant arguments
+                                    `(progn
+                                       ,@(loop for c across value
+                                               collect `(setf (aref .string.
+                                                                    .pos.) ,c)
+                                               collect `(incf .pos.)))
+                                    `(sb!impl::string-dispatch
+                                         (#!+sb-unicode
+                                          (simple-array character (*))
+                                          (simple-array base-char (*))
+                                          t)
+                                         ,var
+                                       (replace .string. ,var :start1 .pos.)
+                                       (incf .pos. (length ,var)))))
+                .string.))
+            lvars)))))
 
 ;;;; CONS accessor DERIVE-TYPE optimizers
 
@@ -1187,41 +1196,49 @@
                                    *
                                    :policy (> speed space))
                 "expand inline"
-                `(let ((index 0)
-                       (find nil)
+                `(let ((find nil)
                        (position nil))
-                   (declare (type index index))
-                   (dolist (i sequence
-                            (if (and end (> end index))
-                                (sequence-bounding-indices-bad-error
-                                 sequence start end)
-                                (values find position)))
-                     (when (and end (>= index end))
-                       (return (values find position)))
-                     (when (>= index start)
-                       (let ((key-i (funcall key i)))
-                         (,',condition (funcall predicate key-i)
-                                       ;; This hack of dealing with non-NIL
-                                       ;; FROM-END for list data by iterating
-                                       ;; forward through the list and keeping
-                                       ;; track of the last time we found a
-                                       ;; match might be more screwy than what
-                                       ;; the user expects, but it seems to be
-                                       ;; allowed by the ANSI standard. (And
-                                       ;; if the user is screwy enough to ask
-                                       ;; for FROM-END behavior on list data,
-                                       ;; turnabout is fair play.)
-                                       ;;
-                                       ;; It's also not enormously efficient,
-                                       ;; calling PREDICATE and KEY more often
-                                       ;; than necessary; but all the
-                                       ;; alternatives seem to have their own
-                                       ;; efficiency problems.
-                                       (if from-end
-                                           (setf find i
-                                                 position index)
-                                           (return (values i index))))))
-                     (incf index))))))
+                   (flet ((bounds-error ()
+                            (sequence-bounding-indices-bad-error sequence start end)))
+                     (if (and end (> start end))
+                         (bounds-error)
+                         (do ((slow sequence (cdr slow))
+                              (fast (cdr sequence) (cddr fast))
+                              (index 0 (+ index 1)))
+                             ((cond ((null slow)
+                                     (if (and end (> end index))
+                                         (bounds-error)
+                                         (return (values find position))))
+                                    ((and end (>= index end))
+                                     (return (values find position)))
+                                    ((eq slow fast)
+                                     (circular-list-error sequence)))
+                              (bug "never"))
+                           (declare (list slow fast))
+                           (when (>= index start)
+                             (let* ((element (car slow))
+                                    (key-i (funcall key element)))
+                               (,',condition (funcall predicate key-i)
+                                             ;; This hack of dealing with non-NIL
+                                             ;; FROM-END for list data by iterating
+                                             ;; forward through the list and keeping
+                                             ;; track of the last time we found a
+                                             ;; match might be more screwy than what
+                                             ;; the user expects, but it seems to be
+                                             ;; allowed by the ANSI standard. (And
+                                             ;; if the user is screwy enough to ask
+                                             ;; for FROM-END behavior on list data,
+                                             ;; turnabout is fair play.)
+                                             ;;
+                                             ;; It's also not enormously efficient,
+                                             ;; calling PREDICATE and KEY more often
+                                             ;; than necessary; but all the
+                                             ;; alternatives seem to have their own
+                                             ;; efficiency problems.
+                                             (if from-end
+                                                 (setf find element
+                                                       position index)
+                                                 (return (values element index)))))))))))))
   (def %find-position-if when)
   (def %find-position-if-not unless))
 
@@ -1351,6 +1368,30 @@
   '(%find-position-vector-macro item sequence
     from-end start end key test))
 
+(deftransform %find-position ((item sequence from-end start end key test)
+                              (character string t t t function function)
+                              *
+                              :policy (> speed space))
+  (if (eq '* (upgraded-element-type-specifier sequence))
+      (let ((form
+             `(sb!impl::string-dispatch ((simple-array character (*))
+                                         (simple-array base-char (*))
+                                         (simple-array nil (*)))
+                  sequence
+                (%find-position item sequence from-end start end key test))))
+        (if (csubtypep (lvar-type sequence) (specifier-type 'simple-string))
+            form
+            ;; Otherwise we'd get three instances of WITH-ARRAY-DATA from
+            ;; %FIND-POSITION.
+            `(with-array-data ((sequence sequence :offset-var offset)
+                               (start start)
+                               (end end)
+                               :check-fill-pointer t)
+               (multiple-value-bind (elt index) ,form
+                 (values elt (when (fixnump index) (- index offset)))))))
+      ;; The type is known exactly, other transforms will take care of it.
+      (give-up-ir1-transform)))
+
 ;;; logic to unravel :TEST, :TEST-NOT, and :KEY options in FIND,
 ;;; POSITION-IF, etc.
 (define-source-transform effective-find-position-test (test test-not)
