@@ -394,8 +394,7 @@
    (move dword-index index)
    (inst shr dword-index 1)
    (inst movss (make-ea-for-float-ref object dword-index offset 4) value)
-   (unless (location= result value)
-     (inst movss result value))))
+   (move result value)))
 
 (define-vop (data-vector-set-c-with-offset/simple-array-single-float)
   (:note "inline array store")
@@ -412,8 +411,7 @@
   (:result-types single-float)
   (:generator 4
    (inst movss (make-ea-for-float-ref object index offset 4) value)
-   (unless (location= result value)
-     (inst movss result value))))
+   (move result value)))
 
 (define-vop (data-vector-ref-with-offset/simple-array-double-float)
   (:note "inline array access")
@@ -460,8 +458,7 @@
   (:result-types double-float)
   (:generator 20
    (inst movsd (make-ea-for-float-ref object index offset 8) value)
-   (unless (location= result value)
-     (inst movsd result value))))
+   (move result value)))
 
 (define-vop (data-vector-set-c-with-offset/simple-array-double-float)
   (:note "inline array store")
@@ -478,8 +475,7 @@
   (:result-types double-float)
   (:generator 19
    (inst movsd (make-ea-for-float-ref object index offset 8) value)
-   (unless (location= result value)
-     (inst movsd result value))))
+   (move result value)))
 
 
 ;;; complex float variants
@@ -497,11 +493,7 @@
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 5
-    (let ((real-tn (complex-single-reg-real-tn value)))
-      (inst movss real-tn (make-ea-for-float-ref object index offset 8)))
-    (let ((imag-tn (complex-single-reg-imag-tn value)))
-      (inst movss imag-tn (make-ea-for-float-ref object index offset 8
-                                                 :complex-offset 4)))))
+    (inst movq value (make-ea-for-float-ref object index offset 8))))
 
 (define-vop (data-vector-ref-c-with-offset/simple-array-complex-single-float)
   (:note "inline array access")
@@ -515,11 +507,7 @@
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 4
-    (let ((real-tn (complex-single-reg-real-tn value)))
-      (inst movss real-tn (make-ea-for-float-ref object index offset 8)))
-    (let ((imag-tn (complex-single-reg-imag-tn value)))
-      (inst movss imag-tn (make-ea-for-float-ref object index offset 8
-                                                 :complex-offset 4)))))
+    (inst movq value (make-ea-for-float-ref object index offset 8))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-single-float)
   (:note "inline array store")
@@ -536,18 +524,8 @@
   (:results (result :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 5
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
-      (inst movss (make-ea-for-float-ref object index offset 8) value-real)
-      (unless (location= value-real result-real)
-        (inst movss result-real value-real)))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
-      (inst movss (make-ea-for-float-ref object index offset 8
-                                         :complex-offset 4)
-            value-imag)
-      (unless (location= value-imag result-imag)
-        (inst movss result-imag value-imag)))))
+    (move result value)
+    (inst movq (make-ea-for-float-ref object index offset 8) value)))
 
 (define-vop (data-vector-set-c-with-offset/simple-array-complex-single-float)
   (:note "inline array store")
@@ -563,18 +541,8 @@
   (:results (result :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 4
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
-      (inst movss (make-ea-for-float-ref object index offset 8) value-real)
-      (unless (location= value-real result-real)
-        (inst movss result-real value-real)))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
-      (inst movss (make-ea-for-float-ref object index offset 8
-                                         :complex-offset 4)
-            value-imag)
-      (unless (location= value-imag result-imag)
-        (inst movss result-imag value-imag)))))
+    (move result value)
+    (inst movq (make-ea-for-float-ref object index offset 8) value)))
 
 (define-vop (data-vector-ref-with-offset/simple-array-complex-double-float)
   (:note "inline array access")
@@ -589,11 +557,7 @@
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 7
-    (let ((real-tn (complex-double-reg-real-tn value)))
-      (inst movsd real-tn (make-ea-for-float-ref object index offset 16 :scale 2)))
-    (let ((imag-tn (complex-double-reg-imag-tn value)))
-      (inst movsd imag-tn (make-ea-for-float-ref object index offset 16 :scale 2
-                                                 :complex-offset 8)))))
+    (inst movapd value (make-ea-for-float-ref object index offset 16 :scale 2))))
 
 (define-vop (data-vector-ref-c-with-offset/simple-array-complex-double-float)
   (:note "inline array access")
@@ -607,11 +571,7 @@
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 6
-    (let ((real-tn (complex-double-reg-real-tn value)))
-      (inst movsd real-tn (make-ea-for-float-ref object index offset 16 :scale 2)))
-    (let ((imag-tn (complex-double-reg-imag-tn value)))
-      (inst movsd imag-tn (make-ea-for-float-ref object index offset 16 :scale 2
-                                                 :complex-offset 8)))))
+    (inst movapd value (make-ea-for-float-ref object index offset 16 :scale 2))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-double-float)
   (:note "inline array store")
@@ -628,19 +588,8 @@
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 20
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
-      (inst movsd (make-ea-for-float-ref object index offset 16 :scale 2)
-            value-real)
-      (unless (location= value-real result-real)
-        (inst movsd result-real value-real)))
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
-      (inst movsd (make-ea-for-float-ref object index offset 16 :scale 2
-                                                 :complex-offset 8)
-            value-imag)
-      (unless (location= value-imag result-imag)
-        (inst movsd result-imag value-imag)))))
+    (inst movapd (make-ea-for-float-ref object index offset 16 :scale 2) value)
+    (move result value)))
 
 (define-vop (data-vector-set-c-with-offset/simple-array-complex-double-float)
   (:note "inline array store")
@@ -656,306 +605,118 @@
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 19
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
-      (inst movsd (make-ea-for-float-ref object index offset 16 :scale 2)
-            value-real)
-      (unless (location= value-real result-real)
-        (inst movsd result-real value-real)))
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
-      (inst movsd (make-ea-for-float-ref object index offset 16 :scale 2
-                                                 :complex-offset 8)
-            value-imag)
-      (unless (location= value-imag result-imag)
-        (inst movsd result-imag value-imag)))))
+    (inst movapd (make-ea-for-float-ref object index offset 16 :scale 2) value)
+    (move result value)))
 
 
 
-;;; unsigned-byte-8
-(macrolet ((define-data-vector-frobs (ptype mov-inst type
-                                            8-bit-tns-p &rest scs)
-  `(progn
-    (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
-      (:translate data-vector-ref-with-offset)
-      (:policy :fast-safe)
-      (:args (object :scs (descriptor-reg))
-             (index :scs (unsigned-reg)))
-      (:info offset)
-      (:arg-types ,ptype positive-fixnum
-                  (:constant (constant-displacement other-pointer-lowtag
-                                                    1 vector-data-offset)))
-      (:results (value :scs ,scs))
-      (:result-types ,type)
-      (:generator 5
-        (inst ,mov-inst value
-              (make-ea :byte :base object :index index :scale 1
-                       :disp (- (+ (* vector-data-offset n-word-bytes)
-                                   offset)
-                                other-pointer-lowtag)))))
-    (define-vop (,(symbolicate "DATA-VECTOR-REF-C-WITH-OFFSET/" ptype))
-      (:translate data-vector-ref-with-offset)
-      (:policy :fast-safe)
-      (:args (object :scs (descriptor-reg)))
-      (:info index offset)
-      (:arg-types ,ptype (:constant low-index)
-                  (:constant (constant-displacement other-pointer-lowtag
-                                                    1 vector-data-offset)))
-      (:results (value :scs ,scs))
-      (:result-types ,type)
-      (:generator 4
-        (inst ,mov-inst value
-              (make-ea :byte :base object
-                       :disp (- (+ (* vector-data-offset n-word-bytes)
-                                   index offset)
-                                other-pointer-lowtag)))))
-    (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
-      (:translate data-vector-set-with-offset)
-      (:policy :fast-safe)
-      (:args (object :scs (descriptor-reg) :to (:eval 0))
-             (index :scs (unsigned-reg) :to (:eval 0))
-             (value :scs ,scs ,@(unless 8-bit-tns-p '(:target rax))))
-      (:info offset)
-      (:arg-types ,ptype positive-fixnum
-                  (:constant (constant-displacement other-pointer-lowtag
-                                                    1 vector-data-offset))
-                  ,type)
-      ,@(unless 8-bit-tns-p
-         '((:temporary (:sc unsigned-reg :offset rax-offset :target result
-                        :from (:argument 2) :to (:result 0))
-            rax)))
-      (:results (result :scs ,scs))
-      (:result-types ,type)
-      (:generator 5
-        ,@(unless 8-bit-tns-p '((move rax value)))
-        (inst mov (make-ea :byte :base object :index index :scale 1
-                           :disp (- (+ (* vector-data-offset n-word-bytes)
-                                       offset)
-                                    other-pointer-lowtag))
-              ,(if 8-bit-tns-p 'value 'al-tn))
-        (move result ,(if 8-bit-tns-p 'value 'rax))))
-    (define-vop (,(symbolicate "DATA-VECTOR-SET-C-WITH-OFFSET/" ptype))
-      (:translate data-vector-set-with-offset)
-      (:policy :fast-safe)
-      (:args (object :scs (descriptor-reg) :to (:eval 0))
-             (value :scs ,scs ,@(unless 8-bit-tns-p '(:target rax))))
-      (:info index offset)
-      (:arg-types ,ptype (:constant low-index)
-                  (:constant (constant-displacement other-pointer-lowtag
-                                                    1 vector-data-offset))
-                  ,type)
-      ,@(unless 8-bit-tns-p
-         '((:temporary (:sc unsigned-reg :offset rax-offset :target result
-                        :from (:argument 2) :to (:result 0))
-            rax)))
-      (:results (result :scs ,scs))
-      (:result-types ,type)
-      (:generator 4
-        ,@(unless 8-bit-tns-p '((move rax value)))
-        (inst mov (make-ea :byte :base object
-                           :disp (- (+ (* vector-data-offset n-word-bytes)
-                                       index offset)
-                                    other-pointer-lowtag))
-              ,(if 8-bit-tns-p 'value 'al-tn))
-        (move result ,(if 8-bit-tns-p 'value 'rax)))))))
-  (define-data-vector-frobs simple-array-unsigned-byte-7 movzx positive-fixnum
-    nil unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-unsigned-byte-8 movzx positive-fixnum
-    nil unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-signed-byte-8 movsx tagged-num
-    nil signed-reg)
+;;; {un,}signed-byte-{8,16,32} and characters
+(macrolet ((define-data-vector-frobs (ptype mov-inst operand-size
+                                            type &rest scs)
+  (let ((n-bytes (ecase operand-size
+                   (:byte 1)
+                   (:word 2)
+                   (:dword 4))))
+    `(progn
+      (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
+        (:translate data-vector-ref-with-offset)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg))
+               (index :scs (unsigned-reg)))
+        (:info offset)
+        (:arg-types ,ptype positive-fixnum
+                    (:constant (constant-displacement other-pointer-lowtag
+                                                      ,n-bytes vector-data-offset)))
+        (:results (value :scs ,scs))
+        (:result-types ,type)
+        (:generator 5
+          (inst ,mov-inst value
+                (make-ea ,operand-size :base object :index index :scale ,n-bytes
+                         :disp (- (+ (* vector-data-offset n-word-bytes)
+                                     (* offset ,n-bytes))
+                                  other-pointer-lowtag)))))
+      (define-vop (,(symbolicate "DATA-VECTOR-REF-C-WITH-OFFSET/" ptype))
+        (:translate data-vector-ref-with-offset)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg)))
+        (:info index offset)
+        (:arg-types ,ptype (:constant low-index)
+                    (:constant (constant-displacement other-pointer-lowtag
+                                                      ,n-bytes vector-data-offset)))
+        (:results (value :scs ,scs))
+        (:result-types ,type)
+        (:generator 4
+          (inst ,mov-inst value
+                (make-ea ,operand-size :base object
+                         :disp (- (+ (* vector-data-offset n-word-bytes)
+                                     (* ,n-bytes index)
+                                     (* ,n-bytes offset))
+                                  other-pointer-lowtag)))))
+      (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
+        (:translate data-vector-set-with-offset)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg) :to (:eval 0))
+               (index :scs (unsigned-reg) :to (:eval 0))
+               (value :scs ,scs :target result))
+        (:info offset)
+        (:arg-types ,ptype positive-fixnum
+                    (:constant (constant-displacement other-pointer-lowtag
+                                                      ,n-bytes vector-data-offset))
+                    ,type)
+        (:results (result :scs ,scs))
+        (:result-types ,type)
+        (:generator 5
+          (inst mov (make-ea ,operand-size :base object :index index :scale ,n-bytes
+                             :disp (- (+ (* vector-data-offset n-word-bytes)
+                                         (* offset ,n-bytes))
+                                      other-pointer-lowtag))
+                (reg-in-size value ,operand-size))
+          (move result value)))
+
+      (define-vop (,(symbolicate "DATA-VECTOR-SET-C-WITH-OFFSET/" ptype))
+        (:translate data-vector-set-with-offset)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg) :to (:eval 0))
+               (value :scs ,scs :target result))
+        (:info index offset)
+        (:arg-types ,ptype (:constant low-index)
+                    (:constant (constant-displacement other-pointer-lowtag
+                                                      ,n-bytes vector-data-offset))
+                    ,type)
+        (:results (result :scs ,scs))
+        (:result-types ,type)
+        (:generator 4
+          (inst mov (make-ea ,operand-size :base object
+                             :disp (- (+ (* vector-data-offset n-word-bytes)
+                                         (* ,n-bytes index)
+                                         (* ,n-bytes offset))
+                                      other-pointer-lowtag))
+                (reg-in-size value ,operand-size))
+          (move result value)))))))
+  (define-data-vector-frobs simple-array-unsigned-byte-7 movzx :byte
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-unsigned-byte-8 movzx :byte
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-signed-byte-8 movsx :byte
+    tagged-num signed-reg)
   (define-data-vector-frobs simple-base-string
-     #!+sb-unicode movzx #!-sb-unicode mov
-     character
-     #!+sb-unicode nil #!-sb-unicode t character-reg))
-
-;;; unsigned-byte-16
-(macrolet ((define-data-vector-frobs (ptype mov-inst type &rest scs)
-    `(progn
-      (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
-        (:translate data-vector-ref-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg))
-               (index :scs (unsigned-reg)))
-        (:info offset)
-        (:arg-types ,ptype positive-fixnum
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      2 vector-data-offset)))
-        (:results (value :scs ,scs))
-        (:result-types ,type)
-        (:generator 5
-          (inst ,mov-inst value
-                (make-ea :word :base object :index index :scale 2
-                         :disp (- (+ (* vector-data-offset n-word-bytes)
-                                     (* offset 2))
-                                  other-pointer-lowtag)))))
-      (define-vop (,(symbolicate "DATA-VECTOR-REF-C-WITH-OFFSET/" ptype))
-        (:translate data-vector-ref-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg)))
-        (:info index offset)
-        (:arg-types ,ptype (:constant low-index)
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      2 vector-data-offset)))
-        (:results (value :scs ,scs))
-        (:result-types ,type)
-        (:generator 4
-          (inst ,mov-inst value
-                (make-ea :word :base object
-                         :disp (- (+ (* vector-data-offset n-word-bytes)
-                                     (* 2 index)
-                                     (* 2 offset))
-                                  other-pointer-lowtag)))))
-      (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
-        (:translate data-vector-set-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg) :to (:eval 0))
-               (index :scs (unsigned-reg) :to (:eval 0))
-               (value :scs ,scs :target eax))
-        (:info offset)
-        (:arg-types ,ptype positive-fixnum
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      2 vector-data-offset))
-                    ,type)
-        (:temporary (:sc unsigned-reg :offset eax-offset :target result
-                         :from (:argument 2) :to (:result 0))
-                    eax)
-        (:results (result :scs ,scs))
-        (:result-types ,type)
-        (:generator 5
-          (move eax value)
-          (inst mov (make-ea :word :base object :index index :scale 2
-                             :disp (- (+ (* vector-data-offset n-word-bytes)
-                                         (* offset 2))
-                                      other-pointer-lowtag))
-                ax-tn)
-          (move result eax)))
-
-      (define-vop (,(symbolicate "DATA-VECTOR-SET-C-WITH-OFFSET/" ptype))
-        (:translate data-vector-set-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg) :to (:eval 0))
-               (value :scs ,scs :target eax))
-        (:info index offset)
-        (:arg-types ,ptype (:constant low-index)
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      2 vector-data-offset))
-                    ,type)
-        (:temporary (:sc unsigned-reg :offset eax-offset :target result
-                         :from (:argument 1) :to (:result 0))
-                    eax)
-        (:results (result :scs ,scs))
-        (:result-types ,type)
-        (:generator 4
-          (move eax value)
-          (inst mov (make-ea :word :base object
-                             :disp (- (+ (* vector-data-offset n-word-bytes)
-                                         (* 2 index)
-                                         (* 2 offset))
-                                      other-pointer-lowtag))
-                ax-tn)
-          (move result eax))))))
-  (define-data-vector-frobs simple-array-unsigned-byte-15 movzx positive-fixnum
-    unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-unsigned-byte-16 movzx positive-fixnum
-    unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-signed-byte-16 movsx tagged-num
-    signed-reg))
-
-(macrolet ((define-data-vector-frobs (ptype mov-inst type &rest scs)
-    `(progn
-      (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
-        (:translate data-vector-ref-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg))
-               (index :scs (unsigned-reg)))
-        (:info offset)
-        (:arg-types ,ptype positive-fixnum
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      4 vector-data-offset)))
-        (:results (value :scs ,scs))
-        (:result-types ,type)
-        (:generator 5
-          (inst ,mov-inst value
-                (make-ea :dword :base object :index index :scale 4
-                         :disp (- (+ (* vector-data-offset n-word-bytes)
-                                     (* offset 4))
-                                  other-pointer-lowtag)))))
-      (define-vop (,(symbolicate "DATA-VECTOR-REF-C-WITH-OFFSET/" ptype))
-        (:translate data-vector-ref-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg)))
-        (:info index offset)
-        (:arg-types ,ptype (:constant low-index)
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      4 vector-data-offset)))
-        (:results (value :scs ,scs))
-        (:result-types ,type)
-        (:generator 4
-          (inst ,mov-inst value
-                (make-ea :dword :base object
-                         :disp (- (+ (* vector-data-offset n-word-bytes)
-                                     (* 4 index)
-                                     (* 4 offset))
-                                  other-pointer-lowtag)))))
-      (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
-        (:translate data-vector-set-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg) :to (:eval 0))
-               (index :scs (unsigned-reg) :to (:eval 0))
-               (value :scs ,scs :target rax))
-        (:info offset)
-        (:arg-types ,ptype positive-fixnum
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      4 vector-data-offset))
-                    ,type)
-        (:temporary (:sc unsigned-reg :offset rax-offset :target result
-                         :from (:argument 2) :to (:result 0))
-                    rax)
-        (:results (result :scs ,scs))
-        (:result-types ,type)
-        (:generator 5
-          (move rax value)
-          (inst mov (make-ea :dword :base object :index index :scale 4
-                                :disp (- (+ (* vector-data-offset n-word-bytes)
-                                            (* offset 4))
-                                         other-pointer-lowtag))
-                eax-tn)
-          (move result rax)))
-
-      (define-vop (,(symbolicate "DATA-VECTOR-SET-C-WITH-OFFSET/" ptype))
-        (:translate data-vector-set-with-offset)
-        (:policy :fast-safe)
-        (:args (object :scs (descriptor-reg) :to (:eval 0))
-               (value :scs ,scs :target rax))
-        (:info index offset)
-        (:arg-types ,ptype (:constant low-index)
-                    (:constant (constant-displacement other-pointer-lowtag
-                                                      4 vector-data-offset))
-                    ,type)
-        (:temporary (:sc unsigned-reg :offset rax-offset :target result
-                         :from (:argument 1) :to (:result 0))
-                    rax)
-        (:results (result :scs ,scs))
-        (:result-types ,type)
-        (:generator 4
-          (move rax value)
-          (inst mov (make-ea :dword :base object
-                             :disp (- (+ (* vector-data-offset n-word-bytes)
-                                         (* 4 index)
-                                         (* 4 offset))
-                                      other-pointer-lowtag))
-                eax-tn)
-          (move result rax))))))
-  (define-data-vector-frobs simple-array-unsigned-byte-32 movzxd positive-fixnum
-    unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-unsigned-byte-31 movzxd positive-fixnum
-    unsigned-reg signed-reg)
-  (define-data-vector-frobs simple-array-signed-byte-32 movsxd tagged-num
-    signed-reg)
+     #!+sb-unicode movzx #!-sb-unicode mov :byte
+     character character-reg)
+  (define-data-vector-frobs simple-array-unsigned-byte-15 movzx :word
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-unsigned-byte-16 movzx :word
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-signed-byte-16 movsx :word
+    tagged-num signed-reg)
+  (define-data-vector-frobs simple-array-unsigned-byte-32 movzxd :dword
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-unsigned-byte-31 movzxd :dword
+    positive-fixnum unsigned-reg signed-reg)
+  (define-data-vector-frobs simple-array-signed-byte-32 movsxd :dword
+    tagged-num signed-reg)
   #!+sb-unicode
-  (define-data-vector-frobs simple-character-string movzxd character
-    character-reg))
+  (define-data-vector-frobs simple-character-string movzxd :dword
+    character character-reg))
 
 
 ;;; These vops are useful for accessing the bits of a vector
