@@ -111,7 +111,12 @@
   (def!constant static-space-end          #x201ff000)
 
   (def!constant dynamic-space-start   #x1000000000)
+  #!-openbsd
   (def!constant dynamic-space-end     #x11ffff0000)
+  #!+openbsd
+  ;; This is lower on OpenBSD to allow SBCL to run under the default
+  ;; 512M data size limit.
+  (def!constant dynamic-space-end     #x101bcf0000)
 
   (def!constant linkage-table-space-start #x20200000)
   (def!constant linkage-table-space-end   #x202ff000)
@@ -121,32 +126,19 @@
 
 ;;;; other miscellaneous constants
 
-(defenum (:suffix -trap :start 8)
-  halt
-  pending-interrupt
-  error
-  cerror
-  breakpoint
-  fun-end-breakpoint
-  single-step-around
-  single-step-before)
-;;; FIXME: It'd be nice to replace all the DEFENUMs with something like
-;;;   (WITH-DEF-ENUM (:START 8)
-;;;     (DEF-ENUM HALT-TRAP)
-;;;     (DEF-ENUM PENDING-INTERRUPT-TRAP)
-;;;     ..)
-;;; for the benefit of anyone doing a lexical search for definitions
-;;; of these symbols.
+(defenum (:start 8)
+  halt-trap
+  pending-interrupt-trap
+  error-trap
+  cerror-trap
+  breakpoint-trap
+  fun-end-breakpoint-trap
+  single-step-around-trap
+  single-step-before-trap)
 
-(defenum (:prefix object-not- :suffix -trap :start 24)
-  list
-  instance)
-
-(defenum (:prefix trace-table-)
-  normal
-  call-site
-  fun-prologue
-  fun-epilogue)
+(defenum (:start 24)
+  object-not-list-trap
+  object-not-instance-trap)
 
 ;;;; static symbols
 
