@@ -18,14 +18,14 @@
 #include "globals.h"
 #include "runtime.h"
 #include <signal.h>
-#include <ucontext.h>
 #include <limits.h>
 #include <mach-o/dyld.h>
 #include "bsd-os.h"
 #include <errno.h>
+#include <dlfcn.h>
 
 char *
-os_get_runtime_executable_path()
+os_get_runtime_executable_path(int external)
 {
     char path[PATH_MAX + 1];
     uint32_t size = sizeof(path);
