@@ -12,8 +12,8 @@ struct alloc_region {
     void  *end_addr; /* pointer to the byte after the last usable byte */
 
     /* These are needed when closing the region. */
-    long  first_page;
-    long  last_page;
+    page_index_t  first_page;
+    page_index_t  last_page;
     void  *start_addr;
 };
 
@@ -22,7 +22,4 @@ extern struct alloc_region  unboxed_region;
 extern generation_index_t from_space, new_space;
 extern struct weak_pointer *weak_pointers;
 
-extern void *current_region_free_pointer;
-extern void *current_region_end_addr;
-
 #endif /*  _GENCGC_ALLOC_REGION_H_ */
