@@ -29,26 +29,30 @@ inline page_index_t find_page_index(void *);
 inline void *page_address(page_index_t);
 int gencgc_handle_wp_violation(void *);
 
+
+/* Note that this structure is also used from Lisp-side in
+ * src/code/room.lisp, and the Lisp-side structure layout is currently
+ * not groveled from C code but hardcoded. Any changes to the
+ * structure layout need to be also made there.
+ *
+ * FIXME: We should probably just define this structure in Lisp, and
+ * output the C version in genesis. -- JES, 2006-12-30.
+ */
 struct page {
-    /* The name of this field is not well-chosen for its actual use.
-     * This is the offset from the start of the page to the start
-     * of the alloc_region which contains/contained it.  It's negative or 0
+    /* This is the offset from the start of the page to the start of
+     * the alloc_region which contains/contained it.
      */
-    long  first_object_offset;
+    unsigned long region_start_offset;
 
     /* the number of bytes of this page that are used. This may be less
      * than the actual bytes used for pages within the current
      * allocation regions. It should be 0 for all unallocated pages (not
      * hard to achieve).
-     *
-     * Currently declared as an unsigned short to make the struct size
-     * smaller. This means that GENCGC-PAGE-SIZE is constrained to fit
-     * inside a short.
      */
+#if PAGE_BYTES > USHRT_MAX
+    unsigned int bytes_used;
+#else
     unsigned short bytes_used;
-
-#if USHRT_MAX < PAGE_BYTES
-#error "PAGE_BYTES too large"
 #endif
 
     unsigned
@@ -62,10 +66,14 @@ struct page {
          * OSes). This is useful for re-scavenging pages that are
          * written during a GC. */
         write_protected_cleared :1,
-        /* the region the page is allocated to: 0 for a free page; 1
-         * for boxed objects; 2 for unboxed objects. If the page is
-         * free the following slots are invalid (well the bytes_used
-         * must be 0). */
+        /*  000 free
+         *  10? boxed data
+         *  11? boxed code
+         *  01? unboxed data
+         *  ??1 open region
+         *
+         * If the page is free the following slots are invalid, except
+         * for the bytes_used which must be zero. */
         allocated :3,
         /* If this page should not be moved during a GC then this flag
          * is set. It's only valid during a GC for allocated pages. */
@@ -88,7 +96,7 @@ struct page {
 /* values for the page.allocated field */
 
 
-extern unsigned page_table_pages;
+extern page_index_t page_table_pages;
 extern struct page *page_table;
 
 
@@ -98,22 +106,25 @@ void sniff_code_object(struct code *code, unsigned long displacement);
 void gencgc_apply_code_fixups(struct code *old_code, struct code *new_code);
 
 long update_dynamic_space_free_pointer(void);
-void gc_alloc_update_page_tables(int unboxed,
-                                 struct alloc_region *alloc_region);
+void gc_alloc_update_page_tables(int page_type_flag, struct alloc_region *alloc_region);
 void gc_alloc_update_all_page_tables(void);
 void gc_set_region_empty(struct alloc_region *region);
 
 /*
  * predicates
  */
+
 static inline boolean
 space_matches_p(lispobj obj, generation_index_t space)
 {
-    page_index_t page_index=(void*)obj - (void *)DYNAMIC_SPACE_START;
-    return ((page_index >= 0)
-            && ((page_index =
-                 ((unsigned long)page_index)/PAGE_BYTES) < page_table_pages)
-            && (page_table[page_index].gen == space));
+    if (obj >= DYNAMIC_SPACE_START) {
+        page_index_t page_index=((pointer_sized_uint_t)obj
+                                 - DYNAMIC_SPACE_START) / PAGE_BYTES;
+        return ((page_index < page_table_pages) &&
+                (page_table[page_index].gen == space));
+    } else {
+        return 0;
+    }
 }
 
 static inline boolean
