@@ -121,6 +121,20 @@ os_context_lr_addr(os_context_t *context)
     return (os_context_register_t *) &context->uc_mcontext->PPC_DARWIN_REGIFY(ss).PPC_DARWIN_REGIFY(lr);
 }
 
+os_context_register_t *
+os_context_ctr_addr(os_context_t *context)
+{
+    /* FIXME: Figure out how to make this happen. */
+    lose("was asked for context Counter (CTR) register, but don't know how");
+}
+
+os_context_register_t *
+os_context_cr_addr(os_context_t *context)
+{
+    /* FIXME: Figure out how to make this happen. */
+    lose("was asked for context Condition (CR) register, but don't know how");
+}
+
 os_context_register_t *
 os_context_pc_addr(os_context_t *context)
 {
