@@ -19,6 +19,12 @@
 #include "machine/npx.h"
 #endif
 
+#if defined(LISP_FEATURE_OPENBSD)
+#include <machine/frame.h>
+#include <machine/npx.h>
+#include <stddef.h>
+#endif
+
 /* KLUDGE: There is strong family resemblance in the signal context
  * stuff in FreeBSD and OpenBSD, but in detail they're different in
  * almost every line of code. It would be nice to find some way to
@@ -156,7 +162,7 @@ int arch_os_thread_init(struct thread *thread) {
     struct segment_descriptor ldt_entry = { 0, 0, SDT_MEMRW, SEL_UPL, 1,
                                             0, 0, 1, 0, 0 };
 
-    set_data_desc_addr(&ldt_entry, (unsigned long) thread);
+    set_data_desc_addr(&ldt_entry, thread);
     set_data_desc_size(&ldt_entry, dynamic_values_bytes);
 
     n = i386_set_ldt(LDT_AUTO_ALLOC, (union descriptor*) &ldt_entry, 1);
@@ -169,8 +175,12 @@ int arch_os_thread_init(struct thread *thread) {
     load_fs(sel);
 
     thread->tls_cookie=n;
+#ifdef LISP_FEATURE_GCC_TLS
+    current_thread = thread;
+#else
     pthread_setspecific(specials,thread);
 #endif
+#endif
 
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
     stack_t sigstack;
@@ -192,8 +202,8 @@ int arch_os_thread_cleanup(struct thread *thread) {
 #if defined(LISP_FEATURE_SB_THREAD)
     int n = thread->tls_cookie;
 
-    /* Set the %%fs register back to 0 and free the the ldt
-     * by setting it to NULL.
+    /* Set the %%fs register back to 0 and free the ldt by setting it
+     * to NULL.
      */
     FSHOW_SIGNAL((stderr, "/ TLS: Freeing LDT %x\n", n));
 
@@ -207,6 +217,14 @@ int arch_os_thread_cleanup(struct thread *thread) {
 #endif /* !LISP_FEATURE_DARWIN */
 
 #if defined(LISP_FEATURE_FREEBSD)
+#if defined(LISP_FEATURE_RESTORE_TLS_SEGMENT_REGISTER_FROM_CONTEXT)
+void
+os_restore_tls_segment_register(os_context_t *context)
+{
+    load_fs(context->uc_mcontext.mc_fs);
+}
+#endif
+
 void
 os_restore_fp_control(os_context_t *context)
 {
@@ -214,8 +232,30 @@ os_restore_fp_control(os_context_t *context)
      * On earlier systems, it is shared in a whole process.
      */
 #if defined(__FreeBSD_version) && __FreeBSD_version >= 500040
-    struct envxmm *ex = (struct envxmm*)(&context->uc_mcontext.mc_fpstate);
-    asm ("fldcw %0" : : "m" (ex->en_cw));
+    struct envxmm *ex = (struct envxmm *)(context->uc_mcontext.mc_fpstate);
+    __asm__ __volatile__ ("fldcw %0" : : "m" (ex->en_cw));
+#endif
+#if defined(LISP_FEATURE_RESTORE_TLS_SEGMENT_REGISTER_FROM_CONTEXT)
+    /* Calling this function here may not be good idea.  Or rename
+     * function name os_restore_fp_control to os_restore_context or
+     * so, to match the behavior?  */
+    os_restore_tls_segment_register(context);
 #endif
 }
 #endif
+
+#if defined(LISP_FEATURE_OPENBSD)
+void
+os_restore_fp_control(os_context_t *context)
+{
+    struct sigframe *frame;
+    union savefpu *fpu;
+
+    frame = (struct sigframe *)((char*)context - offsetof(struct sigframe, sf_sc));
+    fpu = frame->sf_fpstate;
+    if (openbsd_use_fxsave)
+        __asm__ __volatile__ ("fldcw %0" : : "m" (fpu->sv_xmm.sv_env.en_cw));
+    else
+        __asm__ __volatile__ ("fldcw %0" : : "m" (fpu->sv_87.sv_env.en_cw));
+}
+#endif
