@@ -430,15 +430,13 @@ catch_exception_raise(mach_port_t exception_port,
         }
         /* At stack guard */
         if (os_trunc_to_page(addr) == CONTROL_STACK_GUARD_PAGE(th)) {
-            protect_control_stack_guard_page(0, th);
-            protect_control_stack_return_guard_page(1, th);
+            lower_thread_control_stack_guard_page(th);
             handler = control_stack_exhausted_handler;
             break;
         }
         /* Return from stack guard */
         if (os_trunc_to_page(addr) == CONTROL_STACK_RETURN_GUARD_PAGE(th)) {
-            protect_control_stack_guard_page(1, th);
-            protect_control_stack_return_guard_page(0, th);
+            reset_thread_control_stack_guard_page(th);
             break;
         }
         /* Regular memory fault */
