@@ -27,4 +27,12 @@ if [ "`grep -c FOO::BAR $TEST_FILESTEM`" != 1 ] ; then
     echo failed DEFPACKAGE-then-PRINT from --eval form
     exit $EXIT_LOSE
 fi
-exit $EXIT_TEST_WIN
\ No newline at end of file
+
+# --script
+run_sbcl --script script-test.lisp --eval foo \
+  < /dev/null > $TEST_FILESTEM
+if [ "`grep -c :SCRIPT-OK $TEST_FILESTEM`" != 1 ] ; then
+   echo "failed --script test"
+   exit $EXIT_LOSE
+fi
+exit $EXIT_TEST_WIN
